@@ -1,10 +1,7 @@
+/* SPDX-License-Identifier: GPL-2.0+ */
 /*
  * (C) Copyright 2009
  * Stefano Babic, DENX Software Engineering, sbabic@denx.de.
- * Copyright (C) 2015-2016 Freescale Semiconductor, Inc.
- * Copyright 2018 NXP
- *
- * SPDX-License-Identifier:	GPL-2.0+
  */
 
 #ifndef __SYS_PROTO_IMX6_
@@ -12,7 +9,6 @@
 
 #include <asm/mach-imx/sys_proto.h>
 #include <asm/arch/iomux.h>
-#include <asm/arch/module_fuse.h>
 
 #define USBPHY_PWD		0x00000000
 
@@ -24,6 +20,14 @@
 int imx6_pcie_toggle_power(void);
 int imx6_pcie_toggle_reset(void);
 
+enum ldo_reg {
+	LDO_ARM,
+	LDO_SOC,
+	LDO_PU,
+};
+
+int set_ldo_voltage(enum ldo_reg ldo, u32 mv);
+
 /**
  * iomuxc_set_rgmii_io_voltage - set voltage level of RGMII/USB pins
  *
