@@ -1,17 +1,15 @@
+/* SPDX-License-Identifier: GPL-2.0+ */
 /*
  * Copyright (C) 2016 Freescale Semiconductor, Inc.
- * Copyright 2017, 2019 NXP
- *
- * SPDX-License-Identifier: GPL-2.0+
  */
 
 #ifndef _MX7ULP_REGS_H_
 #define _MX7ULP_REGS_H_
 
-#define ARCH_MXC
-
 #include <linux/sizes.h>
 
+#define ARCH_MXC
+
 #define ROM_SW_INFO_ADDR        0x000001E8
 
 #define CAAM_SEC_SRAM_BASE      (0x26000000)
@@ -187,10 +185,10 @@
 #define USDHC0_RBASE	((AIPS2_BASE + (AIPS2_SLOT_SIZE * USDHC0_AIPS2_SLOT)))
 #define USDHC1_RBASE	((AIPS2_BASE + (AIPS2_SLOT_SIZE * USDHC1_AIPS2_SLOT)))
 
-#define DSI_RBASE	((AIPS3_BASE + (AIPS3_SLOT_SIZE * DSI_AIPS3_SLOT)))
 #define LCDIF_RBASE	((AIPS3_BASE + (AIPS3_SLOT_SIZE * LCDIF_AIPS3_SLOT)))
 #define MXS_LCDIF_BASE	LCDIF_RBASE
 
+
 #define SNVS_BASE	((AIPS2_BASE + (AIPS2_SLOT_SIZE * SNVS_AIPS2_SLOT)))
 #define SNVS_LP_LPCR	(SNVS_BASE + 0x38)
 
@@ -968,10 +966,15 @@
 
 #define SNVS_LPCR_DPEN				(0x20)
 #define SNVS_LPCR_SRTC_ENV			(0x1)
+#define SNVS_BASE_REVB				(0x41070000)
+#define SNVS_HPSR_REVB				(SNVS_BASE_REVB + 0x14)
 
-#include <asm/mach-imx/regs-lcdif.h>
+#define SRC_BASE_ADDR			CMC1_RBASE
+#define IRAM_BASE_ADDR			OCRAM_0_BASE
+#define IOMUXC_BASE_ADDR		IOMUXC1_RBASE
 
 #if !(defined(__KERNEL_STRICT_NAMES) || defined(__ASSEMBLY__))
+#include <asm/mach-imx/regs-lcdif.h>
 
 #include <asm/types.h>
 
@@ -1146,10 +1149,6 @@ struct usbphy_regs {
 	u32	usb1_pfda_ctrl1_tog;		/* 0x14c */
 };
 
-
-#define	is_boot_from_usb(void)		(!(readl(USB_PHY0_BASE_ADDR) & (1<<20)))
-#define	disconnect_from_pc(void)	writel(0x0, USBOTG0_RBASE + 0x140)
-
 struct bootrom_sw_info {
 	u8 reserved_1;
 	u8 boot_dev_instance;
@@ -1162,6 +1161,11 @@ struct bootrom_sw_info {
 	u32 reserved_3[3];
 };
 
+#include <stdbool.h>
+bool is_usb_boot(void);
+#define is_boot_from_usb  is_usb_boot
+#define	disconnect_from_pc(void)	writel(0x0, USBOTG0_RBASE + 0x140)
+
 #endif
 
 #endif /* _MX7ULP_REGS_H_*/
