@@ -41,6 +41,8 @@
 #include "fsl_caam_internal.h"
 #include "fsl/desc_constr.h"
 #include <fsl_caam.h>
+#include <cpu_func.h>
+#include <asm/global_data.h>
 
 DECLARE_GLOBAL_DATA_PTR;
 
@@ -76,7 +78,11 @@ struct jr_data_st {
 /*
  * Global variables
  */
+#if defined(CONFIG_SPL_BUILD)
+static struct jr_data_st g_jrdata = {0};
+#else
 static struct jr_data_st g_jrdata = {0, 0, 0xFFFFFFFF};
+#endif
 
 static u8 skeymod[] = {
 	0x0f, 0x0e, 0x0d, 0x0c, 0x0b, 0x0a, 0x09, 0x08,
@@ -141,16 +147,16 @@ u32 caam_decap_blob(u32 plain_text, u32 blob_addr, u32 size)
 	init_job_desc(decap_desc, 0);
 	append_load(decap_desc, PTR2CAAMDMA(skeymod), key_sz,
 		    LDST_CLASS_2_CCB | LDST_SRCDST_BYTE_KEY);
-	append_seq_in_ptr_intlen(decap_desc, blob_addr, size + CAAM_PAD_LEN, 0);
+	append_seq_in_ptr_intlen(decap_desc, blob_addr, size + 48, 0);
 	append_seq_out_ptr_intlen(decap_desc, plain_text, size, 0);
 	append_operation(decap_desc, OP_TYPE_DECAP_PROTOCOL | OP_PCLID_BLOB);
 
 	flush_dcache_range((uintptr_t)blob_addr & ALIGN_MASK,
 			   ((uintptr_t)blob_addr & ALIGN_MASK)
-			    + ROUND(size + CAAM_PAD_LEN, ARCH_DMA_MINALIGN));
+			    + ROUND(2 * size, ARCH_DMA_MINALIGN));
 	flush_dcache_range((uintptr_t)plain_text & ALIGN_MASK,
 			   (plain_text & ALIGN_MASK)
-			   + ROUND(size, ARCH_DMA_MINALIGN));
+			   + ROUND(2 * size, ARCH_DMA_MINALIGN));
 
 	/* Run descriptor with result written to blob buffer */
 	ret = do_job(decap_desc);
@@ -186,15 +192,15 @@ u32 caam_gen_blob(u32 plain_data_addr, u32 blob_addr, u32 size)
 	append_load(encap_desc, PTR2CAAMDMA(skeymod), key_sz,
 		    LDST_CLASS_2_CCB | LDST_SRCDST_BYTE_KEY);
 	append_seq_in_ptr_intlen(encap_desc, plain_data_addr, size, 0);
-	append_seq_out_ptr_intlen(encap_desc, PTR2CAAMDMA(blob), size + CAAM_PAD_LEN, 0);
+	append_seq_out_ptr_intlen(encap_desc, PTR2CAAMDMA(blob), size + 48, 0);
 	append_operation(encap_desc, OP_TYPE_ENCAP_PROTOCOL | OP_PCLID_BLOB);
 
 	flush_dcache_range((uintptr_t)plain_data_addr & ALIGN_MASK,
 			   (plain_data_addr & ALIGN_MASK)
-			   + ROUND(size, ARCH_DMA_MINALIGN));
+			   + ROUND(2 * size, ARCH_DMA_MINALIGN));
 	flush_dcache_range((uintptr_t)blob & ALIGN_MASK,
 			   ((uintptr_t)blob & ALIGN_MASK)
-			   + ROUND(size + CAAM_PAD_LEN, ARCH_DMA_MINALIGN));
+			   + ROUND(2 * size, ARCH_DMA_MINALIGN));
 
 	ret = do_job(encap_desc);
 
@@ -250,6 +256,8 @@ void caam_open(void)
 
 	/* switch on the clock */
 #ifndef CONFIG_ARCH_IMX8
+	u32 init_mask;
+
 	caam_clock_enable();
 #endif
 
@@ -270,7 +278,8 @@ void caam_open(void)
 
 	/* Check if the RNG is already instantiated */
 	temp_reg = __raw_readl(CAAM_RDSTA);
-	if (temp_reg == (RDSTA_IF0 | RDSTA_IF1 | RDSTA_SKVN)) {
+	init_mask = RDSTA_IF0 | RDSTA_IF1 | RDSTA_SKVN;
+	if ((temp_reg & init_mask) == init_mask) {
 		printf("RNG already instantiated 0x%X\n", temp_reg);
 		return;
 	}
@@ -365,11 +374,18 @@ static void kick_trng(u32 ent_delay)
 	val = (ent_delay << BS_TRNG_ENT_DLY) | samples;
 	__raw_writel(val, CAAM_RTSDCTL);
 
-	/* min. freq. count, equal to 1/2 of the entropy sample length */
+	/*
+	 * Recommended margins (min,max) for freq. count:
+	 *   freq_mul = RO_freq / TRNG_clk_freq
+	 *   rtfrqmin = (ent_delay x freq_mul) >> 1;
+	 *   rtfrqmax = (ent_delay x freq_mul) << 3;
+	 * Given current deployments of CAAM in i.MX SoCs, and to simplify
+	 * the configuration, we consider [1,16] to be a safe interval
+	 * for the freq_mul and the limits of the interval are used to compute
+	 * rtfrqmin, rtfrqmax
+	 */
 	__raw_writel(ent_delay >> 1, CAAM_RTFRQMIN);
-
-	/* max. freq. count, equal to 32 times the entropy sample length */
-	__raw_writel(ent_delay << 5, CAAM_RTFRQMAX);
+	__raw_writel(ent_delay << 7, CAAM_RTFRQMAX);
 
 	__raw_writel((retries << 16) | lrun_max, CAAM_RTSCMISC);
 	__raw_writel(poker_max, CAAM_RTPKRMAX);
@@ -405,11 +421,13 @@ static void kick_trng(u32 ent_delay)
 /*
  *  Descriptors to instantiate SH0, SH1, load the keys
  */
+#ifndef CONFIG_ARCH_IMX8
 static const u32 rng_inst_sh0_desc[] = {
 	/* Header, don't setup the size */
 	CAAM_HDR_CTYPE | CAAM_HDR_ONE | CAAM_HDR_START_INDEX(0),
 	/* Operation instantiation (sh0) */
-	CAAM_PROTOP_CTYPE | CAAM_C1_RNG | ALGO_RNG_SH(0) | ALGO_RNG_INSTANTIATE,
+	CAAM_PROTOP_CTYPE | CAAM_C1_RNG | ALGO_RNG_SH(0) | ALGO_RNG_PR |
+		ALGO_RNG_INSTANTIATE,
 };
 
 static const u32 rng_inst_sh1_desc[] = {
@@ -420,7 +438,7 @@ static const u32 rng_inst_sh1_desc[] = {
 	CAAM_C0_LOAD_IMM | CAAM_DST_CLEAR_WRITTEN | sizeof(u32),
 	0x00000001,
 	/* Operation instantiation (sh1) */
-	CAAM_PROTOP_CTYPE | CAAM_C1_RNG | ALGO_RNG_SH(1)
+	CAAM_PROTOP_CTYPE | CAAM_C1_RNG | ALGO_RNG_SH(1) | ALGO_RNG_PR
 		| ALGO_RNG_INSTANTIATE,
 };
 
@@ -434,6 +452,7 @@ static const u32 rng_inst_load_keys[] = {
 	/* Generate the Key */
 	CAAM_PROTOP_CTYPE | CAAM_C1_RNG | BM_ALGO_RNG_SK | ALGO_RNG_GENERATE,
 };
+#endif
 
 static void do_inst_desc(u32 *desc, u32 status)
 {
@@ -549,15 +568,15 @@ static int do_job(u32 *desc)
 			   ((uintptr_t)desc & ALIGN_MASK)
 			   + ROUND(DESC_MAX_SIZE, ARCH_DMA_MINALIGN));
 
-	flush_dcache_range((uintptr_t)g_jrdata.outrings & ALIGN_MASK,
-			   ((uintptr_t)g_jrdata.outrings & ALIGN_MASK)
-			   + ROUND(DESC_MAX_SIZE, ARCH_DMA_MINALIGN));
-
 	/* Inform HW that a new JR is available */
 	__raw_writel(1, CAAM_IRJAR0);
 	while (__raw_readl(CAAM_ORSFR0) == 0)
 		;
 
+	flush_dcache_range((uintptr_t)g_jrdata.outrings & ALIGN_MASK,
+			   ((uintptr_t)g_jrdata.outrings & ALIGN_MASK)
+			   + ROUND(DESC_MAX_SIZE, ARCH_DMA_MINALIGN));
+
 	if (PTR2CAAMDMA(desc) == g_jrdata.outrings[0].desc) {
 		ret = g_jrdata.outrings[0].status;
 	} else {
@@ -602,22 +621,15 @@ static int do_cfg_jrqueue(void)
 		g_jrdata.status = RING_RELOC_INIT;
 	} else {
 		u32 align_idx = 0;
-		u32 *addr;
 
-#if defined(CONFIG_SPL_BUILD) && CONFIG_VAL(SYS_MALLOC_F_LEN)
-		ulong maddr = (ulong)malloc(DESC_MAX_SIZE * 2 + 8);
-		addr = (u32*)ALIGN(maddr, 8);
-#else
-		addr = g_jrdata.raw_addr;
-#endif
 		/* Ensure 64bits buffers addresses alignment */
-		if ((uintptr_t)addr & 0x7)
+		if ((uintptr_t)g_jrdata.raw_addr & 0x7)
 			align_idx = 1;
 		g_jrdata.inrings  = (struct inring_entry *)
-				    (&addr[align_idx]);
+				    (&g_jrdata.raw_addr[align_idx]);
 		g_jrdata.outrings = (struct outring_entry *)
-				    (&addr[align_idx + 2]);
-		g_jrdata.desc = (u32 *)(&addr[align_idx + 4]);
+				    (&g_jrdata.raw_addr[align_idx + 2]);
+		g_jrdata.desc = (u32 *)(&g_jrdata.raw_addr[align_idx + 4]);
 		g_jrdata.status = RING_EARLY_INIT;
 	}
 
