@@ -1,16 +1,17 @@
+/* SPDX-License-Identifier: GPL-2.0+ */
 /*
  * Copyright (C) 2016 Freescale Semiconductor, Inc.
  * Copyright 2018 NXP
- * Copyright 2019 Sony Home Entertainment & Sound Products Inc.
- *
- * SPDX-License-Identifier:	GPL-2.0+
+ * Copyright 2019,2022 Sony Corporation
  */
 #include <common.h>
+#include <command.h>
 #include <asm/arch/sys_proto.h>
 #include <linux/errno.h>
 #include <asm/io.h>
 #include <stdbool.h>
 #include <mmc.h>
+#include <env.h>
 
 static int check_mmc_autodetect(void)
 {
@@ -34,7 +35,6 @@ void board_late_mmc_env_init(void)
 {
 	char cmd[32];
 	char mmcblk[32];
-	char mmcblk_verity[32];
 	u32 dev_no = mmc_get_env_dev();
 
 	if (!check_mmc_autodetect())
@@ -46,11 +46,6 @@ void board_late_mmc_env_init(void)
 	snprintf(mmcblk, sizeof(mmcblk), "/dev/mmcblk%dp2 rootwait rw",
 		mmc_map_to_kernel_blk(dev_no));
 	env_set("mmcroot", mmcblk);
-	
-	/* Set mmcverity env*/
-	sprintf(mmcblk_verity, "/dev/mmcblk%dp3",
-		mmc_map_to_kernel_blk(dev_no));	
-	env_set("mmcverity", mmcblk_verity);
 
 	snprintf(cmd, sizeof(cmd), "mmc dev %d", dev_no);
 	run_command(cmd, 0);
