@@ -1,11 +1,12 @@
 /*
- * Copyright 2017 NXP
+ * Copyright 2017,2019 NXP
  *
  * SPDX-License-Identifier:	GPL-2.0+
  */
 #include <common.h>
 #include <i2c.h>
 #include <time.h>
+#include <linux/delay.h>
 #include "tcpc.h"
 
 #ifdef DEBUG
@@ -231,6 +232,20 @@ int tcpc_clear_alert(struct tcpc_port *port, uint16_t clear_mask)
 	return 0;
 }
 
+int tcpc_fault_status_mask(struct tcpc_port *port, uint8_t fault_mask)
+{
+	int err = 0;
+
+	if (!port)
+		return -EINVAL;
+
+	err = dm_i2c_write(port->i2c_dev, TCPC_FAULT_STATUS_MASK, &fault_mask, 1);
+	if (err)
+		tcpc_log(port, "%s dm_i2c_write failed, err %d\n", __func__, err);
+
+	return err;
+}
+
 int tcpc_send_command(struct tcpc_port *port, uint8_t command)
 {
 	int err;
@@ -295,7 +310,7 @@ int tcpc_setup_dfp_mode(struct tcpc_port *port)
 	enum typec_cc_state state;
 	int ret;
 
-	if (port == NULL)
+	if ((port == NULL) || (port->i2c_dev == NULL))
 		return -EINVAL;
 
 	if (tcpc_pd_sink_check_charging(port)) {
@@ -362,7 +377,7 @@ int tcpc_setup_ufp_mode(struct tcpc_port *port)
 	enum typec_cc_state state;
 	int ret;
 
-	if (port == NULL)
+	if ((port == NULL) || (port->i2c_dev == NULL))
 		return -EINVAL;
 
 	/* Check if the PD charge is working. If not, need to configure CC role for UFP */
@@ -497,7 +512,7 @@ static int tcpc_pd_transmit_message(struct tcpc_port *port, struct pd_message *m
 {
 	int ret;
 	uint8_t valb;
-	uint16_t val;
+	uint16_t val = 0;
 
 	if (port == NULL)
 		return -EINVAL;
@@ -549,12 +564,12 @@ static int tcpc_pd_transmit_message(struct tcpc_port *port, struct pd_message *m
 	return ret;
 }
 
-static void tcpc_log_source_caps(struct tcpc_port *port, uint32_t *caps, unsigned int capcount)
+static void tcpc_log_source_caps(struct tcpc_port *port, struct pd_message *msg, unsigned int capcount)
 {
 	int i;
 
 	for (i = 0; i < capcount; i++) {
-		u32 pdo = caps[i];
+		u32 pdo = msg->payload[i];
 		enum pd_pdo_type type = pdo_type(pdo);
 
 		tcpc_log(port, "PDO %d: type %d, ",
@@ -597,7 +612,7 @@ static void tcpc_log_source_caps(struct tcpc_port *port, uint32_t *caps, unsigne
 	}
 }
 
-static int tcpc_pd_select_pdo(uint32_t *caps, uint32_t capcount, uint32_t max_snk_mv, uint32_t max_snk_ma)
+static int tcpc_pd_select_pdo(struct pd_message *msg, uint32_t capcount, uint32_t max_snk_mv, uint32_t max_snk_ma)
 {
 	unsigned int i, max_mw = 0, max_mv = 0;
 	int ret = -EINVAL;
@@ -607,7 +622,7 @@ static int tcpc_pd_select_pdo(uint32_t *caps, uint32_t capcount, uint32_t max_sn
 	 * the board's voltage limits. Prefer PDO providing exp
 	 */
 	for (i = 0; i < capcount; i++) {
-		u32 pdo = caps[i];
+		u32 pdo = msg->payload[i];
 		enum pd_pdo_type type = pdo_type(pdo);
 		unsigned int mv, ma, mw;
 
@@ -637,7 +652,7 @@ static int tcpc_pd_select_pdo(uint32_t *caps, uint32_t capcount, uint32_t max_sn
 }
 
 static int tcpc_pd_build_request(struct tcpc_port *port,
-										uint32_t *caps,
+										struct pd_message *msg,
 										uint32_t capcount,
 										uint32_t max_snk_mv,
 										uint32_t max_snk_ma,
@@ -651,11 +666,11 @@ static int tcpc_pd_build_request(struct tcpc_port *port,
 	int index;
 	u32 pdo;
 
-	index = tcpc_pd_select_pdo(caps, capcount, max_snk_mv, max_snk_ma);
+	index = tcpc_pd_select_pdo(msg, capcount, max_snk_mv, max_snk_ma);
 	if (index < 0)
 		return -EINVAL;
 
-	pdo = caps[index];
+	pdo = msg->payload[index];
 	type = pdo_type(pdo);
 
 	if (type == PDO_TYPE_FIXED)
@@ -724,12 +739,11 @@ static void tcpc_pd_sink_process(struct tcpc_port *port)
 			if (msgtype != PD_DATA_SOURCE_CAP)
 				continue;
 
-			uint32_t *caps = (uint32_t *)&msg.payload;
 			uint32_t rdo = 0;
 
-			tcpc_log_source_caps(port, caps, objcnt);
+			tcpc_log_source_caps(port, &msg, objcnt);
 
-			tcpc_pd_build_request(port, caps, objcnt,
+			tcpc_pd_build_request(port, &msg, objcnt,
 				port->cfg.max_snk_mv, port->cfg.max_snk_ma,
 				port->cfg.max_snk_mw, port->cfg.op_snk_mv,
 				&rdo);
@@ -833,9 +847,18 @@ static int tcpc_pd_sink_disable(struct tcpc_port *port)
 	}
 
 	if ((valb & TCPC_POWER_STATUS_VBUS_PRES) && (valb & TCPC_POWER_STATUS_SINKING_VBUS)) {
-		dm_i2c_read(port->i2c_dev, TCPC_POWER_CTRL, (uint8_t *)&valb, 1);
+		err = dm_i2c_read(port->i2c_dev, TCPC_POWER_CTRL, (uint8_t *)&valb, 1);
+		if (err) {
+			tcpc_log(port, "%s dm_i2c_read failed, err %d\n", __func__, err);
+			return -EIO;
+		}
+
 		valb &= ~TCPC_POWER_CTRL_AUTO_DISCH_DISCO; /* disable AutoDischargeDisconnect */
-		dm_i2c_write(port->i2c_dev, TCPC_POWER_CTRL, (const uint8_t *)&valb, 1);
+		err = dm_i2c_write(port->i2c_dev, TCPC_POWER_CTRL, (const uint8_t *)&valb, 1);
+		if (err) {
+			tcpc_log(port, "%s dm_i2c_write failed, err %d\n", __func__, err);
+			return -EIO;
+		}
 
 		tcpc_disable_sink_vbus(port);
 	}
@@ -894,12 +917,27 @@ static int tcpc_pd_sink_init(struct tcpc_port *port)
 		tcpc_log(port, "TCPC wrong state for dead battery, err = %d, CC = 0x%x\n",
 			err, state);
 		return -EPERM;
-	} else
+	} else {
+		err = tcpc_set_plug_orientation(port, pol);
+		if (err) {
+			tcpc_log(port, "TCPC set plug orientation failed, err = %d\n", err);
+			return err;
+		}
 		port->pd_state = ATTACHED;
+	}
+
+	err = dm_i2c_read(port->i2c_dev, TCPC_POWER_CTRL, (uint8_t *)&valb, 1);
+	if (err) {
+		tcpc_log(port, "%s dm_i2c_read failed, err %d\n", __func__, err);
+		return -EIO;
+	}
 
-	dm_i2c_read(port->i2c_dev, TCPC_POWER_CTRL, (uint8_t *)&valb, 1);
 	valb &= ~TCPC_POWER_CTRL_AUTO_DISCH_DISCO; /* disable AutoDischargeDisconnect */
-	dm_i2c_write(port->i2c_dev, TCPC_POWER_CTRL, (const uint8_t *)&valb, 1);
+	err = dm_i2c_write(port->i2c_dev, TCPC_POWER_CTRL, (const uint8_t *)&valb, 1);
+	if (err) {
+		tcpc_log(port, "%s dm_i2c_write failed, err %d\n", __func__, err);
+		return -EIO;
+	}
 
 	if (port->cfg.switch_setup_func)
 		port->cfg.switch_setup_func(port);
@@ -970,7 +1008,7 @@ int tcpc_init(struct tcpc_port *port, struct tcpc_port_config config, ss_mux_sel
 	}
 
 	dm_i2c_read(port->i2c_dev, TCPC_POWER_STATUS, &valb, 1);
-	tcpc_debug_log("POWER STATUS: 0x%x\n", valb);
+	tcpc_debug_log(port, "POWER STATUS: 0x%x\n", valb);
 
 	/* Clear AllRegistersResetToDefault */
 	valb = 0x80;
@@ -1004,6 +1042,9 @@ int tcpc_init(struct tcpc_port *port, struct tcpc_port_config config, ss_mux_sel
 		tcpc_pd_sink_disable(port);
 	}
 
+	/* Mask all fault status */
+	tcpc_fault_status_mask(port, 0);
+
 	tcpc_clear_alert(port, 0xffff);
 
 	tcpc_print_log(port);
