@@ -1,5 +1,5 @@
 /*
- * opyright 2017 NXP
+ * Copyright 2020 NXP
  *
  * SPDX-License-Identifier:	GPL-2.0+
  */
@@ -7,34 +7,19 @@
 #ifndef IMX8MQ_EVK_ANDROID_H
 #define IMX8MQ_EVK_ANDROID_H
 
-#define CONFIG_CMD_READ
-
-#define CONFIG_ANDROID_AB_SUPPORT
-#define CONFIG_AVB_SUPPORT
-#define CONFIG_SUPPORT_EMMC_RPMB
-#define CONFIG_SYSTEM_RAMDISK_SUPPORT
-#define CONFIG_AVB_FUSE_BANK_SIZEW 0
-#define CONFIG_AVB_FUSE_BANK_START 0
-#define CONFIG_AVB_FUSE_BANK_END 0
-#define CONFIG_FASTBOOT_LOCK
 #define FSL_FASTBOOT_FB_DEV "mmc"
 
 #ifdef CONFIG_SYS_MALLOC_LEN
 #undef CONFIG_SYS_MALLOC_LEN
-#define CONFIG_SYS_MALLOC_LEN           (96 * SZ_1M)
+#define CONFIG_SYS_MALLOC_LEN           (32 * SZ_1M)
 #endif
 
-#define CONFIG_ANDROID_RECOVERY
-
-#define CONFIG_CMD_BOOTA
-#define CONFIG_SUPPORT_RAW_INITRD
-#define CONFIG_SERIAL_TAG
-
 #undef CONFIG_EXTRA_ENV_SETTINGS
 #undef CONFIG_BOOTCOMMAND
 
 #define CONFIG_EXTRA_ENV_SETTINGS		\
 	"splashpos=m,m\0"			\
+	"splashimage=0x50000000\0"		\
 	"fdt_high=0xffffffffffffffff\0"		\
 	"initrd_high=0xffffffffffffffff\0"	\
 
@@ -42,13 +27,28 @@
 #ifdef CONFIG_FLASH_MCUFIRMWARE_SUPPORT
 #define ANDROID_MCU_FRIMWARE_DEV_TYPE DEV_MMC
 #define ANDROID_MCU_FIRMWARE_START 0x500000
-#define ANDROID_MCU_FIRMWARE_SIZE  0x40000
+#define ANDROID_MCU_OS_PARTITION_SIZE 0x40000
+#define ANDROID_MCU_FIRMWARE_SIZE  0x20000
 #define ANDROID_MCU_FIRMWARE_HEADER_STACK 0x20020000
 #endif
 
-#ifdef CONFIG_FSL_CAAM_KB
-#undef CONFIG_FSL_CAAM_KB
+#ifdef CONFIG_DUAL_BOOTLOADER
+#define CONFIG_SYS_SPL_PTE_RAM_BASE    0x41580000
+
+#ifdef CONFIG_IMX_TRUSTY_OS
+#define BOOTLOADER_RBIDX_OFFSET  0x1E000
+#define BOOTLOADER_RBIDX_START   0x1F000
+#define BOOTLOADER_RBIDX_LEN     0x08
+#define BOOTLOADER_RBIDX_INITVAL 0
+#endif
+
+#endif
+
+#ifdef CONFIG_IMX_TRUSTY_OS
+#define AVB_RPMB
+#define KEYSLOT_HWPARTITION_ID 2
+#define KEYSLOT_BLKS             0x1FFF
+#define NS_ARCH_ARM64 1
 #endif
-#define AVB_AB_I_UNDERSTAND_LIBAVB_AB_IS_DEPRECATED
 
 #endif /* IMX8MQ_EVK_ANDROID_H */
