@@ -24,7 +24,7 @@
 #undef CONFIG_LOADADDR
 #define CONFIG_LOADADDR			0x80080000
 #undef CONFIG_SYS_INIT_SP_ADDR
-#define CONFIG_SYS_INIT_SP_ADDR		0x80200000
+#define CONFIG_SYS_INIT_SP_ADDR		0x81200000
 
 #undef CONFIG_REQUIRE_SERIAL_CONSOLE
 #undef CONFIG_IMX_SMMU
@@ -32,12 +32,19 @@
 #undef CONFIG_FASTBOOT_USB_DEV
 #define CONFIG_FASTBOOT_USB_DEV 0 /* Use OTG port, not typec port */
 
-/* This needs to be stay same in iomem in domu.cfg */
-#ifdef SC_IPC_CH
-#undef SC_IPC_CH
-#endif
-#define SC_IPC_CH			0x5d1d0000
+#ifdef CONFIG_SPL_BUILD
+#undef CONFIG_SPL_BSS_START_ADDR
+#undef CONFIG_SYS_SPL_MALLOC_START
+#undef CONFIG_MALLOC_F_ADDR
+#undef CONFIG_SPL_TEXT_BASE
+#undef CONFIG_SPL_STACK
 
-#define AVB_AB_I_UNDERSTAND_LIBAVB_AB_IS_DEPRECATED
+#define CONFIG_MALLOC_F_ADDR		0x80100000
+#define CONFIG_SYS_SPL_MALLOC_START	0x80200000
+#define CONFIG_SPL_BSS_START_ADDR	0x80300000
+#define CONFIG_SPL_STACK		0x80400000
+
+#define CONFIG_SYS_SPL_PTE_RAM_BASE	0x80500000
+#endif
 
 #endif /* IMX8QM_MEK_ANDROID_AUTO_XEN_H */
