@@ -1,15 +1,15 @@
+/* SPDX-License-Identifier: GPL-2.0+ */
 /*
  * Copyright (C) 2015 Freescale Semiconductor, Inc.
  *
  * Configuration settings for the Freescale i.MX7.
- *
- * SPDX-License-Identifier:	GPL-2.0+
  */
 
 #ifndef __MX7_COMMON_H
 #define __MX7_COMMON_H
 
 #include <linux/sizes.h>
+#include <linux/stringify.h>
 #include <asm/arch/imx-regs.h>
 #include <asm/mach-imx/gpio.h>
 
@@ -30,37 +30,31 @@
 
 #define CONFIG_LOADADDR                 0x80800000
 
-/* allow to overwrite serial and ethaddr */
-#define CONFIG_ENV_OVERWRITE
-#define CONFIG_CONS_INDEX               1
-#define CONFIG_BAUDRATE                 115200
-
 /* Miscellaneous configurable options */
 #define CONFIG_SYS_CBSIZE		2048
 #define CONFIG_SYS_MAXARGS		32
 #define CONFIG_SYS_BARGSIZE		CONFIG_SYS_CBSIZE
 
-#ifndef CONFIG_SYS_DCACHE_OFF
-#endif
-
 /* UART */
-#define CONFIG_MXC_UART
 
-/* MMC */
-#define CONFIG_BOUNCE_BUFFER
-#define CONFIG_FSL_ESDHC
-#define CONFIG_FSL_USDHC
-
-/* Fuses */
-#define CONFIG_MXC_OCOTP
+/* NET PHY */
+#define PHY_ANEG_TIMEOUT 20000
 
 #define CONFIG_ARMV7_SECURE_BASE	0x00900000
 
-#define CONFIG_ARMV7_PSCI_1_0
+#ifdef CONFIG_SPL_BUILD
+#define CONFIG_SPL_DRIVERS_MISC_SUPPORT
+#endif
 
-/* Secure boot (HAB) support */
-#ifdef CONFIG_SECURE_BOOT
-#define CONFIG_CSF_SIZE			0x4000
+/*
+ * If we have defined the OPTEE ram size and not OPTEE it means that we were
+ * launched by OPTEE, because of that we shall skip all the low level
+ * initialization since it was already done by ATF or OPTEE
+ */
+#if (CONFIG_OPTEE_TZDRAM_SIZE != 0)
+#ifndef CONFIG_OPTEE
+#define CONFIG_SKIP_LOWLEVEL_INIT
+#endif
 #endif
 
 #ifdef CONFIG_IMX_OPTEE
@@ -68,4 +62,5 @@
 #else
 #define TEE_ENV "tee=no\0"
 #endif
+
 #endif
