@@ -1,9 +1,9 @@
+/* SPDX-License-Identifier: GPL-2.0+ */
 /*
  *  Copyright (C) 2012 Samsung Electronics
  *  Lukasz Majewski <l.majewski@samsung.com>
- *  Copyright 2019 Sony Home Entertainment & Sound Products Inc.
  *
- * SPDX-License-Identifier:	GPL-2.0+
+ *  Copyright 2019,2022 Sony Corporation
  */
 
 #ifndef __G_DOWNLOAD_H_
@@ -40,13 +40,14 @@ int g_dnl_board_usb_cable_connected(void);
 int g_dnl_register(const char *s);
 void g_dnl_unregister(void);
 void g_dnl_set_serialnumber(char *);
+void g_dnl_set_product(const char *s);
 
 bool g_dnl_detach(void);
 void g_dnl_trigger_detach(void);
 void g_dnl_clear_detach(void);
-
 bool g_dnl_requested(void);
 void g_dnl_trigger_start(void);
 int run_usb_dnl_gadget(int usbctrl_index, char *usb_dnl_gadget);
+int board_usb_gadget_port_auto(void);
 
 #endif /* __G_DOWNLOAD_H_ */
