@@ -247,7 +247,7 @@ static int km_get_version(int32_t *version)
 
     rc = km_send_request(KM_GET_VERSION, NULL, 0);
     if (rc < 0) {
-        trusty_error("failed to send km version request", rc);
+        trusty_error("failed (%d) to send km version request", rc);
         return rc;
     }
 
@@ -290,6 +290,9 @@ int km_tipc_init(struct trusty_ipc_dev *dev)
         return TRUSTY_ERR_GENERIC;
     }
 
+    /* mark as initialized */
+    initialized = true;
+
     return TRUSTY_ERR_NONE;
 }
 
@@ -395,6 +398,10 @@ end:
 int trusty_set_attestation_key(const uint8_t *key, uint32_t key_size,
                                keymaster_algorithm_t algorithm)
 {
+    if (!initialized) {
+        trusty_error("Keymaster TIPC client not initialized!\n");
+        return -1;
+    }
     return trusty_send_attestation_data(KM_SET_ATTESTATION_KEY, key, key_size,
                                         algorithm);
 }
@@ -403,10 +410,37 @@ int trusty_append_attestation_cert_chain(const uint8_t *cert,
                                          uint32_t cert_size,
                                          keymaster_algorithm_t algorithm)
 {
+    if (!initialized) {
+        trusty_error("Keymaster TIPC client not initialized!\n");
+        return -1;
+    }
     return trusty_send_attestation_data(KM_APPEND_ATTESTATION_CERT_CHAIN,
                                         cert, cert_size, algorithm);
 }
 
+int trusty_set_attestation_key_enc(const uint8_t *key, uint32_t key_size,
+                               keymaster_algorithm_t algorithm)
+{
+    if (!initialized) {
+        trusty_error("Keymaster TIPC client not initialized!\n");
+        return -1;
+    }
+    return trusty_send_attestation_data(KM_SET_ATTESTATION_KEY_ENC, key, key_size,
+                                        algorithm);
+}
+
+int trusty_append_attestation_cert_chain_enc(const uint8_t *cert,
+                                         uint32_t cert_size,
+                                         keymaster_algorithm_t algorithm)
+{
+    if (!initialized) {
+        trusty_error("Keymaster TIPC client not initialized!\n");
+        return -1;
+    }
+    return trusty_send_attestation_data(KM_APPEND_ATTESTATION_CERT_CHAIN_ENC,
+                                        cert, cert_size, algorithm);
+}
+
 int trusty_atap_get_ca_request(const uint8_t *operation_start,
                                uint32_t operation_start_size,
                                uint8_t **ca_request_p,
@@ -477,3 +511,192 @@ int trusty_atap_read_uuid_str(char **uuid_p)
     }
     return rc;
 }
+
+int trusty_get_mppubk(uint8_t *mppubk, uint32_t *size)
+{
+    int rc = TRUSTY_ERR_GENERIC;
+    struct km_get_mppubk_resp resp;
+
+    if (!initialized) {
+        trusty_error("Keymaster TIPC client not initialized!\n");
+        return -1;
+    }
+
+    rc = km_send_request(KM_GET_MPPUBK, NULL, 0);
+    if (rc < 0) {
+        trusty_error("%s: failed (%d) to send km mppubk request\n", __func__, rc);
+        return rc;
+    }
+
+    rc = km_read_raw_response(KM_GET_MPPUBK, &resp, sizeof(resp));
+    if (rc < 0) {
+        trusty_error("%s: failed (%d) to read km mppubk response\n", __func__, rc);
+        return rc;
+    }
+
+    if (resp.data_size != 64) {
+        trusty_error("%s: Wrong mppubk size!\n", __func__);
+        return TRUSTY_ERR_GENERIC;
+    } else {
+        *size = resp.data_size;
+    }
+
+    memcpy(mppubk, resp.data, resp.data_size);
+    return TRUSTY_ERR_NONE;
+}
+
+int trusty_verify_secure_unlock(uint8_t *unlock_credential,
+                                uint32_t credential_size,
+                                uint8_t *serial, uint32_t serial_size)
+{
+    int rc = TRUSTY_ERR_GENERIC;
+    uint8_t *req = NULL;
+    uint32_t req_size = 0;
+
+    if (!initialized) {
+        trusty_error("Keymaster TIPC client not initialized!\n");
+        return -1;
+    }
+
+    struct km_secure_unlock_data secure_unlock_data = {
+        .serial_size = serial_size,
+        .serial_data = serial,
+        .credential_size = credential_size,
+        .credential_data = unlock_credential,
+    };
+
+    rc = km_secure_unlock_data_serialize(&secure_unlock_data,
+                                             &req, &req_size);
+
+    if (rc < 0) {
+        trusty_error("failed (%d) to serialize request\n", rc);
+        goto end;
+    }
+    rc = km_do_tipc(KM_VERIFY_SECURE_UNLOCK, req, req_size, NULL, NULL);
+
+end:
+    if (req) {
+        trusty_free(req);
+    }
+    return rc;
+}
+
+char *get_serial(void);
+int trusty_set_attestation_id(void)
+{
+    uint8_t *req = NULL, *tmp = NULL;
+    uint32_t req_size = 0;
+    char *serial = NULL;
+    int rc;
+
+    req = trusty_calloc(1024, 1); // 1024 bytes buffer should be enough.
+    tmp = req;
+
+    /* fill in the device ids */
+    /* brand */
+    rc = km_attestation_id_data_serialize((uint8_t *)CONFIG_ATTESTATION_ID_BRAND,
+                                          strlen(CONFIG_ATTESTATION_ID_BRAND),
+                                          &tmp, &req_size);
+    if (rc < 0) {
+	trusty_error("%s: failed (%d) to set id brand.\n", __func__, rc);
+        goto end;
+    }
+
+    /* device */
+    rc = km_attestation_id_data_serialize((uint8_t *)CONFIG_ATTESTATION_ID_DEVICE,
+                                          strlen(CONFIG_ATTESTATION_ID_DEVICE),
+                                          &tmp, &req_size);
+    if (rc < 0) {
+	trusty_error("%s: failed (%d) to set id device.\n", __func__, rc);
+        goto end;
+    }
+
+    /* product */
+    rc = km_attestation_id_data_serialize((uint8_t *)CONFIG_ATTESTATION_ID_PRODUCT,
+                                          strlen(CONFIG_ATTESTATION_ID_PRODUCT),
+                                          &tmp, &req_size);
+    if (rc < 0) {
+	trusty_error("%s: failed (%d) to set id product.\n", __func__, rc);
+        goto end;
+    }
+
+    /* serial number, bail out when fail because it's a MUST. */
+    serial = get_serial();
+    if (serial != NULL) {
+        rc = km_attestation_id_data_serialize((uint8_t *)serial, 16, &tmp, &req_size);
+        if (rc < 0) {
+	    trusty_error("%s: failed (%d) to set id serial.\n", __func__, rc);
+            goto end;
+        }
+    } else {
+        trusty_error("%s: failed to get serial number.\n", __func__);
+        goto end;
+    }
+
+    /* IMEI */
+    rc = km_attestation_id_data_serialize((uint8_t *)CONFIG_ATTESTATION_ID_IMEI,
+                                          strlen(CONFIG_ATTESTATION_ID_IMEI),
+                                          &tmp, &req_size);
+    if (rc < 0) {
+	trusty_error("%s: failed (%d) to set id IMEI.\n", __func__, rc);
+        goto end;
+    }
+
+    /* MEID */
+    rc = km_attestation_id_data_serialize((uint8_t *)CONFIG_ATTESTATION_ID_MEID,
+                                          strlen(CONFIG_ATTESTATION_ID_MEID),
+                                          &tmp, &req_size);
+    if (rc < 0) {
+	trusty_error("%s: failed (%d) to set id MEID.\n", __func__, rc);
+        goto end;
+    }
+
+    /* manufacturer */
+    rc = km_attestation_id_data_serialize((uint8_t *)CONFIG_ATTESTATION_ID_MANUFACTURER,
+                                          strlen(CONFIG_ATTESTATION_ID_MANUFACTURER),
+                                          &tmp, &req_size);
+    if (rc < 0) {
+	trusty_error("%s: failed (%d) to set id manufacturer.\n", __func__, rc);
+        goto end;
+    }
+
+    /* model */
+    rc = km_attestation_id_data_serialize((uint8_t *)CONFIG_ATTESTATION_ID_MODEL,
+                                          strlen(CONFIG_ATTESTATION_ID_MODEL),
+                                          &tmp, &req_size);
+    if (rc < 0) {
+	trusty_error("%s: failed (%d) to set id model.\n", __func__, rc);
+        goto end;
+    }
+
+    rc = km_do_tipc(KM_SET_ATTESTATION_IDS, req, req_size, NULL, NULL);
+
+end:
+    if (req) {
+        trusty_free(req);
+    }
+    return rc;
+}
+
+int trusty_set_boot_patch_level(uint32_t boot_patch_level)
+{
+    if (!initialized) {
+        trusty_error("Keymaster TIPC client not initialized!\n");
+        return -1;
+    }
+
+    uint8_t *req = NULL;
+    uint32_t req_size = 0;
+    int rc;
+
+    req = trusty_calloc(4, 1); // 4 bytes should be enough.
+    memcpy(req, &boot_patch_level, sizeof(uint32_t));
+    req_size = sizeof(uint32_t);
+
+    rc = km_do_tipc(KM_CONFIGURE_BOOT_PATCHLEVEL, req, req_size, NULL, NULL);
+
+    if (req) {
+        trusty_free(req);
+    }
+    return rc;
+}
