@@ -3,6 +3,7 @@
  * Copyright 2016-2018 Sony Corporation.
  * Copyright 2019 Sony Video & Sound Products Inc.
  * Copyright 2019 Sony Home Entertainment & Sound Products Inc.
+ * Copyright 2022 Sony Corporation
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -79,6 +80,12 @@ enum {
 	NVP_SYSTEM_PARAM_NUM,
 };
 
+/* system parameter 2 area index */
+enum {
+	NVP_SYSTEM_PARAM2_MAC = 14,
+	NVP_SYSTEM_PARAM2_NUM = 16,
+};
+
 /* flag information */
 #define NVP_FLAG_BMD_DIAG                       0x67414964
 #define NVP_FLAG_BMD_FWUPDATE                   0x70555766
@@ -116,11 +123,16 @@ typedef struct {
 	unsigned char   serialno[16];
 	unsigned char   ship[32];
 	unsigned char   serialno2[64];
-	unsigned int    tst_flag;
+	unsigned int	tst_flag;
 	unsigned int	gty_flag;
 	unsigned int    ins_flag;
 } nvp_system_parameter_t;
 
+/* boot system parameter 2 typedef */
+typedef struct {
+	unsigned char   mac[6];
+} nvp_system_parameter2_t;
+
 
 /********************/
 /*@ Local variables */
@@ -130,6 +142,7 @@ typedef struct {
 static unsigned int nvp_area_sector_table[NVP_ID_NUM] = {
 	 0,							/* area 00 table : boot parameter		*/
 	 1,							/* area 01 table : system parameter		*/
+	 3,							/* area 03 table : system parameter 2		*/
 	 8,							/* area 08 table : boot image			*/
 	 9,							/* area 09 table : hold image			*/
 	10,							/* area 10 table : low battery image	*/
@@ -177,6 +190,31 @@ static nvp_param_info_t nvp_system_param_table[NVP_SYSTEM_PARAM_NUM] = {
 	{&nvp_system_param.ins_flag,     4},          /* Install flag         */
 };
 
+/* system parameter 2 */
+static nvp_system_parameter2_t nvp_system_param2;
+
+static nvp_param_info_t nvp_system_param2_table[NVP_SYSTEM_PARAM2_NUM] = {
+/*   +---------------------------------------------- address to variable  */
+/*   |                                +------------- size [byte]          */
+/*   |                                |                                   */
+	{NULL,                         704},          /* Alexa setting fix value */
+	{NULL,                         160},          /* lyric info           */
+	{NULL,                         520},          /* empr icv             */
+	{NULL,                           4},          /* FWUP result          */
+	{NULL,                          64},          /* USB Descriptor Manufacture String */
+	{NULL,                        8192},          /* slacker id file      */
+	{NULL,                          32},          /* slacker id file (Continued) */
+	{NULL,                          64},          /* USB Descriptor Product String */
+	{NULL,                        8192},          /* AWS key              */
+	{NULL,                        8192},          /* AWS key (Continued)  */
+	{NULL,                        8192},          /* AWS key (Continued)  */
+	{NULL,                        8192},          /* AWS key (Continued)  */
+	{NULL,                         256},          /* AWS key (Continued)  */
+	{NULL,                          20},          /* firmware update Vendor Identification */
+	{&nvp_system_param2.mac[0],      6},          /* Ethernet MAC address */
+	{NULL,                          64},          /* firmware update Product Identification */
+};
+
 static int nvp_initialized = 0;
 
 
@@ -271,6 +309,33 @@ int nvp_emmc_init(void)
 		}
 	}
 
+	/**********************************************/
+	/* get system parameters 2 (Large sector type */
+	/**********************************************/
+	memset(&nvp_system_param2, 0, sizeof(nvp_system_param2));
+
+	sector = NVP_TABLE_SECTOR + nvp_area_sector_table[NVP_ID_SYSTEM_PARAM2];
+	ret = nvp_read(sector, (void *)buf, BLK_SIZE);
+	if (ret < 0) {
+		return ret;
+	}
+
+	sector_buf_large = (unsigned short *)buf;
+
+	for (i = 0; i < NVP_SYSTEM_PARAM2_NUM; i++) {
+		if ((NVP_DATA_SECTOR_MIN <= (sector_buf_large[i] * NVP_DATA_SECTOR_SIZE_LARGE)) &&
+		    ((sector_buf_large[i] * NVP_DATA_SECTOR_SIZE_LARGE) <= NVP_DATA_SECTOR_MAX)) {
+			if (nvp_system_param2_table[i].param == NULL)
+				continue;
+
+			nvp_get_param(nvp_system_param2_table[i].param, nvp_system_param2_table[i].size, sector_buf_large[i] * NVP_DATA_SECTOR_SIZE_LARGE);
+
+			printf("[NVP] system param 2 %d: %02X %02X %02X %02X\n",
+                               i, *((unsigned char *)(unsigned char *)nvp_system_param2_table[i].param), *((unsigned char *)((unsigned char *)nvp_system_param2_table[i].param + 1)),
+                               *((unsigned char *)((unsigned char *)nvp_system_param2_table[i].param + 2)), *((unsigned char *)((unsigned char *)nvp_system_param2_table[i].param + 3)));
+		}
+	}
+
 	nvp_initialized = 1;
 
 	return ret;
@@ -528,6 +593,30 @@ int nvp_emmc_get_pwd(char *p_pwd, size_t size)
 	return 0;
 }
 
+/*-----------------------------------------------------------------------------
+ * Function : nvp_emmc_get_mac
+ * Abstruct : Get ethernet MAC address
+ * Input    : p_mac
+ * Return   : 0:success -1:fail
+ *----------------------------------------------------------------------------*/
+int nvp_emmc_get_mac(unsigned char *p_mac, size_t size)
+{
+	int ret;
+
+	if (nvp_initialized == 0) {
+		ret = nvp_emmc_init();
+		if (ret < 0) {
+			return ret;
+		}
+	}
+
+	if (size > nvp_system_param2_table[NVP_SYSTEM_PARAM2_MAC].size)
+		return -1;
+
+	memcpy(p_mac, &nvp_system_param2.mac[0], size);
+	return 0;
+}
+
 /*******************/
 /*@ Local routines */
 /*******************/
@@ -571,7 +660,7 @@ static int nvp_get_param(char *dst, int size, int sector)
 /* MMC/NAND I/F */
 /*==============*/
 static struct blk_desc *nvp_dev = NULL;
-static disk_partition_t nvp_part = {0};
+static struct disk_partition nvp_part = {0};
 
 /*-----------------------------------------------------------------------------
  * Function : nvp_get_dev
@@ -582,7 +671,7 @@ static disk_partition_t nvp_part = {0};
  *----------------------------------------------------------------------------*/
 static int nvp_get_dev(void)
 {
-	disk_partition_t info;
+	struct disk_partition info;
 	int ret;
 	int mmc_dev_no;
 
@@ -600,7 +689,7 @@ static int nvp_get_dev(void)
 		return -ENODEV;
 	}
 
-	memcpy(&nvp_part, &info, sizeof(disk_partition_t));
+	memcpy(&nvp_part, &info, sizeof(struct disk_partition));
 
 	return 0;
 }
