@@ -1,10 +1,7 @@
+// SPDX-License-Identifier: GPL-2.0+
 /*
  * (C) Copyright 2002 ELTEC Elektronik AG
  * Frank Gottschling <fgottschling@eltec.de>
- *
- * Copyright 2019 Sony Home Entertainment & Sound Products Inc.
- *
- * SPDX-License-Identifier:	GPL-2.0+
  */
 
 /*
@@ -68,10 +65,16 @@
  */
 
 #include <common.h>
+#include <command.h>
+#include <cpu_func.h>
+#include <env.h>
 #include <fdtdec.h>
+#include <gzip.h>
+#include <log.h>
 #include <version.h>
 #include <malloc.h>
 #include <video.h>
+#include <asm/global_data.h>
 #include <linux/compiler.h>
 
 #if defined(CONFIG_VIDEO_MXS)
@@ -115,10 +118,6 @@
 #define VIDEO_DATA_FORMAT	(pGD->gdfIndex)
 #define VIDEO_FB_ADRS		(pGD->frameAdrs)
 
-#if defined(CONFIG_SONY_KEEP_LOGO)
-#define CONFIG_HIDE_LOGO_VERSION
-#endif /* CONFIG_SONY_KEEP_LOGO */
-
 /*
  * Console device
  */
@@ -162,14 +161,6 @@ void console_cursor(int state);
 #define VIDEO_LOGO_LUT_OFFSET	BMP_LOGO_OFFSET
 #define VIDEO_LOGO_COLORS	BMP_LOGO_COLORS
 
-#ifdef CONFIG_VIDEO_ICX_LOGO
-#include <mmc.h>
-#include <part.h>
-#include <errno.h>
-#include <ext4fs.h>
-#include <icx_logo.h>
-#endif /* CONFIG_VIDEO_ICX_LOGO */
-
 #else  /* CONFIG_VIDEO_BMP_LOGO */
 #define LINUX_LOGO_WIDTH	80
 #define LINUX_LOGO_HEIGHT	80
@@ -782,7 +773,7 @@ static void parse_putc(const char c)
 		break;
 
 	case '\n':		/* next line */
-		if (console_col || (!console_col && nl))
+		if (console_col || nl)
 			console_newline(1);
 		nl = 1;
 		break;
@@ -1312,6 +1303,10 @@ next_run:
 			break;
 		}
 	}
+
+	if (cfb_do_flush_cache)
+		flush_cache(VIDEO_FB_ADRS, VIDEO_SIZE);
+
 	return 0;
 error:
 	printf("Error: Too much encoded pixel data, validate your bitmap\n");
@@ -1322,17 +1317,7 @@ error:
 /*
  * Display the BMP file located at address bmp_image.
  */
-#ifdef CONFIG_VIDEO_ICX_LOGO
 int video_display_bitmap(ulong bmp_image, int x, int y)
-{
-	return video_display_bitmap_to_fb(bmp_image, x, y,
-					 (ulong)video_fb_address);
-}
-
-int video_display_bitmap_to_fb(ulong bmp_image, int x, int y, ulong fb_addr)
-#else
-int video_display_bitmap(ulong bmp_image, int x, int y)
-#endif
 {
 	ushort xcount, ycount;
 	uchar *fb;
@@ -1349,15 +1334,6 @@ int video_display_bitmap(ulong bmp_image, int x, int y)
 	ulong len;
 #endif
 
-#ifdef CONFIG_VIDEO_ICX_LOGO
-	if (fb_addr == 0) {
-		pr_err("[boot logo] Cannot display at fb_addr = 0\n");
-		return 1;
-	}
-
-	printf("[boot logo] Display fb 0x%lx\n", fb_addr);
-#endif
-
 	WATCHDOG_RESET();
 
 	if (!((bmp->header.signature[0] == 'B') &&
@@ -1458,15 +1434,9 @@ int video_display_bitmap(ulong bmp_image, int x, int y)
 		height = VIDEO_VISIBLE_ROWS - y;
 
 	bmap = (uchar *) bmp + le32_to_cpu(bmp->header.data_offset);
-#ifdef CONFIG_VIDEO_ICX_LOGO
-	fb = (uchar *)(fb_addr +
-		      ((y + height - 1) * VIDEO_LINE_LEN) +
-		      x * VIDEO_PIXEL_SIZE);
-#else
 	fb = (uchar *) (video_fb_address +
 			((y + height - 1) * VIDEO_LINE_LEN) +
 			x * VIDEO_PIXEL_SIZE);
-#endif
 
 #ifdef CONFIG_VIDEO_BMP_RLE8
 	if (compression == BMP_BI_RLE8) {
@@ -1743,7 +1713,8 @@ static void logo_black(void)
 			1);
 }
 
-static int do_clrlogo(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
+static int do_clrlogo(struct cmd_tbl *cmdtp, int flag, int argc,
+		      char *const argv[])
 {
 	if (argc != 1)
 		return cmd_usage(cmdtp);
@@ -1884,181 +1855,6 @@ static void plot_logo_or_black(void *screen, int x, int y, int black)
 #endif
 }
 
-#ifdef CONFIG_VIDEO_ICX_LOGO
-static int load_logo_bmp_file(const char *part_name,
-			      const char *file_name, const char *env_name)
-{
-	struct blk_desc *dev_desc = NULL;
-	disk_partition_t info;
-	int ret;
-	loff_t file_len, read_len;
-	char *buf;
-
-	dev_desc = blk_get_dev("mmc", mmc_get_env_dev());
-	if (!dev_desc) {
-		pr_err("[boot logo] blk_get_dev failed.\n");
-		return -1;
-	}
-
-	ret = part_get_info_by_name(dev_desc, part_name, &info);
-	if (ret < 0) {
-		pr_err("[boot logo] part_get_info_by_name failed.\n");
-		return -1;
-	}
-	env_set_hex("boot_logo_part", (ulong)ret);
-	ext4fs_set_blk_dev(dev_desc, &info);
-
-	ret = ext4fs_mount(info.size);
-	if (!ret) {
-		pr_err("[boot logo] ext4fs_mount failed.\n");
-		return -1;
-	}
-
-	ret = ext4fs_open(file_name, &file_len);
-	if (ret < 0) {
-		pr_err("[boot logo] file %s not found.\n", file_name);
-		return -1;
-	}
-
-	buf = (char *)malloc(file_len);
-	if (!buf) {
-		pr_err("[boot logo] malloc failed.\n");
-		ext4fs_close();
-		return -1;
-	}
-
-	env_set_addr(env_name, (void *)buf);
-
-	ret = ext4fs_read(buf, 0, file_len, &read_len);
-	ext4fs_close();
-	if (ret == -1) {
-		pr_err("[boot logo] ext4fs_read failed.\n");
-		return -1;
-	}
-
-	printf("[boot logo] Display %s\n", file_name);
-
-	return 0;
-}
-
-static int load_logo_bmp_offset(const char *part_name, const ulong offset,
-				const ulong count, const char *env_name)
-{
-	struct blk_desc *dev_desc = NULL;
-	int ret;
-	disk_partition_t part_info;
-	void *buf;
-
-	dev_desc = blk_get_dev("mmc", mmc_get_env_dev());
-	if (!dev_desc) {
-		pr_err("[boot logo] blk_get_dev failed.\n");
-		return -1;
-	}
-
-	ret = part_get_info_by_name(dev_desc, part_name, &part_info);
-	if (ret < 0) {
-		pr_err("[boot logo] part_get_info_by_name %s failed.\n",
-		       part_name);
-		return -1;
-	}
-
-	buf = (void *)malloc(CONFIG_SYS_VIDEO_LOGO_MAX_SIZE);
-	if (!buf) {
-		pr_err("[boot logo] malloc failed.\n");
-		return -1;
-	}
-
-	if (count * dev_desc->blksz > CONFIG_SYS_VIDEO_LOGO_MAX_SIZE) {
-		pr_err("[boot logo] bmp count too large. Supported to %d bytes\n",
-		       CONFIG_SYS_VIDEO_LOGO_MAX_SIZE);
-		return -1;
-	}
-
-	if (count + offset > part_info.size) {
-		pr_err("[boot logo] Read out of range\n");
-		return -1;
-	}
-
-	ret = blk_dread(dev_desc, part_info.start + offset, count, buf);
-	if (ret != count) {
-		pr_err("[boot logo] Error reading blocks\n");
-		return -1;
-	}
-
-	env_set_addr(env_name, (void *)buf);
-
-	printf("[boot logo] Display logo from %s offset %ld\n",
-	       part_name, offset);
-
-	return 0;
-}
-
-static int check_logo_pos(const int x, const int y)
-{
-	if (((x != BMP_ALIGN_CENTER) && (x > VIDEO_VISIBLE_COLS)) || x < 0) {
-		pr_err("[boot logo] invalid pos x %d\n", x);
-		return -1;
-	}
-	if (((y != BMP_ALIGN_CENTER) && (y > VIDEO_VISIBLE_COLS)) || y < 0) {
-		pr_err("[boot logo] invalid pos y %d\n", y);
-		return -1;
-	}
-	return 0;
-}
-
-static int display_bitmap_addr(const ulong addr, const ulong fb_addr)
-{
-	int ret;
-
-	ret = check_logo_pos(video_logo_xpos, video_logo_ypos);
-	if (ret)
-		return ret;
-
-	if (!addr)
-		return -1;
-
-	if (!fb_addr)
-		return -1;
-
-	ret = video_display_bitmap_to_fb(addr,
-					 video_logo_xpos, video_logo_ypos,
-					 fb_addr);
-
-	return ret;
-}
-
-int display_icx_logo_file(const char *part_name, const char *file_name)
-{
-	int ret;
-	const char *env_name = "icx_logo_file";
-
-	ret = load_logo_bmp_file(part_name, file_name, env_name);
-	if (ret)
-		return ret;
-
-	ret = display_bitmap_addr(env_get_hex(env_name, 0),
-				  (ulong)video_fb_address);
-
-	return ret;
-}
-
-int display_icx_logo_offset(const char *part_name,
-			    const ulong offset, const ulong count)
-{
-	int ret;
-	const char *env_name = "icx_logo_offset";
-
-	ret = load_logo_bmp_offset(part_name, offset, count, env_name);
-	if (ret)
-		return ret;
-
-	ret = display_bitmap_addr(env_get_hex(env_name, 0),
-				  (ulong)video_fb_address);
-
-	return ret;
-}
-#endif /* CONFIG_VIDEO_ICX_LOGO */
-
 static void *video_logo(void)
 {
 	char info[128];
@@ -2086,25 +1882,6 @@ static void *video_logo(void)
 	}
 #endif /* CONFIG_SPLASH_SCREEN */
 
-#ifdef CONFIG_VIDEO_ICX_LOGO
-	ret = splash_screen_prepare();
-	if (ret < 0)
-		return video_fb_address;
-
-	/* Load and splash kernel logo */
-	memset(CONFIG_KERNEL_LOGO_ADDR, 0, pGD->memSize);
-	load_logo_bmp_file(env_get("logo_part"), env_get("kernel_logo"),
-			   "kernel_logo_addr");
-	display_bitmap_addr(env_get_hex("kernel_logo_addr", 0),
-			    (ulong)CONFIG_KERNEL_LOGO_ADDR);
-
-	ret = display_icx_logo_file(env_get("logo_part"), env_get("boot_logo"));
-	if (ret == 0) {
-		video_logo_height = 0;
-		return ((void *)(video_fb_address));
-	}
-#endif /* CONFIG_VIDEO_ICX_LOGO */
-
 	logo_plot(video_fb_address, video_logo_xpos, video_logo_ypos);
 
 #ifdef CONFIG_SPLASH_SCREEN_ALIGN
@@ -2206,9 +1983,7 @@ static void *video_logo(void)
 
 static int cfb_fb_is_in_dram(void)
 {
-	bd_t *bd = gd->bd;
-#if defined(CONFIG_ARM) || defined(CONFIG_NDS32) || \
-defined(CONFIG_SANDBOX) || defined(CONFIG_X86)
+	struct bd_info *bd = gd->bd;
 	ulong start, end;
 	int i;
 
@@ -2219,11 +1994,7 @@ defined(CONFIG_SANDBOX) || defined(CONFIG_X86)
 		    (ulong)video_fb_address < end)
 			return 1;
 	}
-#else
-	if ((ulong)video_fb_address >= bd->bi_memstart &&
-	    (ulong)video_fb_address < bd->bi_memstart + bd->bi_memsize)
-		return 1;
-#endif
+
 	return 0;
 }
 
